# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal

import httpx

from ..types import (
    util_token_counter_params,
    util_transform_request_params,
    util_get_supported_openai_params_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.util_token_counter_response import UtilTokenCounterResponse
from ..types.util_transform_request_response import UtilTransformRequestResponse

__all__ = ["UtilsResource", "AsyncUtilsResource"]


class UtilsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UtilsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return UtilsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UtilsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return UtilsResourceWithStreamingResponse(self)

    def get_supported_openai_params(
        self,
        *,
        model: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Returns supported openai params for a given litellm model name

        e.g.

        `gpt-4` vs `gpt-3.5-turbo`

        Example curl:

        ```
        curl -X GET --location 'http://localhost:4000/utils/supported_openai_params?model=gpt-3.5-turbo-16k'         --header 'Authorization: Bearer sk-1234'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/utils/supported_openai_params",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"model": model}, util_get_supported_openai_params_params.UtilGetSupportedOpenAIParamsParams
                ),
            ),
            cast_to=object,
        )

    def token_counter(
        self,
        *,
        model: str,
        messages: Optional[Iterable[object]] | NotGiven = NOT_GIVEN,
        prompt: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UtilTokenCounterResponse:
        """
        Token Counter

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/utils/token_counter",
            body=maybe_transform(
                {
                    "model": model,
                    "messages": messages,
                    "prompt": prompt,
                },
                util_token_counter_params.UtilTokenCounterParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UtilTokenCounterResponse,
        )

    def transform_request(
        self,
        *,
        call_type: Literal[
            "embedding",
            "aembedding",
            "completion",
            "acompletion",
            "atext_completion",
            "text_completion",
            "image_generation",
            "aimage_generation",
            "moderation",
            "amoderation",
            "atranscription",
            "transcription",
            "aspeech",
            "speech",
            "rerank",
            "arerank",
            "_arealtime",
            "create_batch",
            "acreate_batch",
            "aretrieve_batch",
            "retrieve_batch",
            "pass_through_endpoint",
            "anthropic_messages",
            "get_assistants",
            "aget_assistants",
            "create_assistants",
            "acreate_assistants",
            "delete_assistant",
            "adelete_assistant",
            "acreate_thread",
            "create_thread",
            "aget_thread",
            "get_thread",
            "a_add_message",
            "add_message",
            "aget_messages",
            "get_messages",
            "arun_thread",
            "run_thread",
            "arun_thread_stream",
            "run_thread_stream",
            "afile_retrieve",
            "file_retrieve",
            "afile_delete",
            "file_delete",
            "afile_list",
            "file_list",
            "acreate_file",
            "create_file",
            "afile_content",
            "file_content",
            "create_fine_tuning_job",
            "acreate_fine_tuning_job",
            "acancel_fine_tuning_job",
            "cancel_fine_tuning_job",
            "alist_fine_tuning_jobs",
            "list_fine_tuning_jobs",
            "aretrieve_fine_tuning_job",
            "retrieve_fine_tuning_job",
            "responses",
            "aresponses",
        ],
        request_body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UtilTransformRequestResponse:
        """
        Transform Request

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/utils/transform_request",
            body=maybe_transform(
                {
                    "call_type": call_type,
                    "request_body": request_body,
                },
                util_transform_request_params.UtilTransformRequestParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UtilTransformRequestResponse,
        )


class AsyncUtilsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUtilsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncUtilsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUtilsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncUtilsResourceWithStreamingResponse(self)

    async def get_supported_openai_params(
        self,
        *,
        model: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Returns supported openai params for a given litellm model name

        e.g.

        `gpt-4` vs `gpt-3.5-turbo`

        Example curl:

        ```
        curl -X GET --location 'http://localhost:4000/utils/supported_openai_params?model=gpt-3.5-turbo-16k'         --header 'Authorization: Bearer sk-1234'
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/utils/supported_openai_params",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"model": model}, util_get_supported_openai_params_params.UtilGetSupportedOpenAIParamsParams
                ),
            ),
            cast_to=object,
        )

    async def token_counter(
        self,
        *,
        model: str,
        messages: Optional[Iterable[object]] | NotGiven = NOT_GIVEN,
        prompt: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UtilTokenCounterResponse:
        """
        Token Counter

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/utils/token_counter",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "messages": messages,
                    "prompt": prompt,
                },
                util_token_counter_params.UtilTokenCounterParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UtilTokenCounterResponse,
        )

    async def transform_request(
        self,
        *,
        call_type: Literal[
            "embedding",
            "aembedding",
            "completion",
            "acompletion",
            "atext_completion",
            "text_completion",
            "image_generation",
            "aimage_generation",
            "moderation",
            "amoderation",
            "atranscription",
            "transcription",
            "aspeech",
            "speech",
            "rerank",
            "arerank",
            "_arealtime",
            "create_batch",
            "acreate_batch",
            "aretrieve_batch",
            "retrieve_batch",
            "pass_through_endpoint",
            "anthropic_messages",
            "get_assistants",
            "aget_assistants",
            "create_assistants",
            "acreate_assistants",
            "delete_assistant",
            "adelete_assistant",
            "acreate_thread",
            "create_thread",
            "aget_thread",
            "get_thread",
            "a_add_message",
            "add_message",
            "aget_messages",
            "get_messages",
            "arun_thread",
            "run_thread",
            "arun_thread_stream",
            "run_thread_stream",
            "afile_retrieve",
            "file_retrieve",
            "afile_delete",
            "file_delete",
            "afile_list",
            "file_list",
            "acreate_file",
            "create_file",
            "afile_content",
            "file_content",
            "create_fine_tuning_job",
            "acreate_fine_tuning_job",
            "acancel_fine_tuning_job",
            "cancel_fine_tuning_job",
            "alist_fine_tuning_jobs",
            "list_fine_tuning_jobs",
            "aretrieve_fine_tuning_job",
            "retrieve_fine_tuning_job",
            "responses",
            "aresponses",
        ],
        request_body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UtilTransformRequestResponse:
        """
        Transform Request

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/utils/transform_request",
            body=await async_maybe_transform(
                {
                    "call_type": call_type,
                    "request_body": request_body,
                },
                util_transform_request_params.UtilTransformRequestParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UtilTransformRequestResponse,
        )


class UtilsResourceWithRawResponse:
    def __init__(self, utils: UtilsResource) -> None:
        self._utils = utils

        self.get_supported_openai_params = to_raw_response_wrapper(
            utils.get_supported_openai_params,
        )
        self.token_counter = to_raw_response_wrapper(
            utils.token_counter,
        )
        self.transform_request = to_raw_response_wrapper(
            utils.transform_request,
        )


class AsyncUtilsResourceWithRawResponse:
    def __init__(self, utils: AsyncUtilsResource) -> None:
        self._utils = utils

        self.get_supported_openai_params = async_to_raw_response_wrapper(
            utils.get_supported_openai_params,
        )
        self.token_counter = async_to_raw_response_wrapper(
            utils.token_counter,
        )
        self.transform_request = async_to_raw_response_wrapper(
            utils.transform_request,
        )


class UtilsResourceWithStreamingResponse:
    def __init__(self, utils: UtilsResource) -> None:
        self._utils = utils

        self.get_supported_openai_params = to_streamed_response_wrapper(
            utils.get_supported_openai_params,
        )
        self.token_counter = to_streamed_response_wrapper(
            utils.token_counter,
        )
        self.transform_request = to_streamed_response_wrapper(
            utils.transform_request,
        )


class AsyncUtilsResourceWithStreamingResponse:
    def __init__(self, utils: AsyncUtilsResource) -> None:
        self._utils = utils

        self.get_supported_openai_params = async_to_streamed_response_wrapper(
            utils.get_supported_openai_params,
        )
        self.token_counter = async_to_streamed_response_wrapper(
            utils.token_counter,
        )
        self.transform_request = async_to_streamed_response_wrapper(
            utils.transform_request,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .member import Member as Member
from .user_roles import UserRoles as UserRoles
from .budget_table import BudgetTable as BudgetTable
from .member_param import MemberParam as MemberParam
from .key_list_params import KeyListParams as KeyListParams
from .file_list_params import FileListParams as FileListParams
from .key_block_params import KeyBlockParams as KeyBlockParams
from .model_info_param import ModelInfoParam as ModelInfoParam
from .org_member_param import OrgMemberParam as OrgMemberParam
from .team_list_params import TeamListParams as TeamListParams
from .user_list_params import UserListParams as UserListParams
from .batch_list_params import BatchListParams as BatchListParams
from .key_delete_params import KeyDeleteParams as KeyDeleteParams
from .key_list_response import KeyListResponse as KeyListResponse
from .key_update_params import KeyUpdateParams as KeyUpdateParams
from .model_list_params import ModelListParams as ModelListParams
from .team_block_params import TeamBlockParams as TeamBlockParams
from .budget_info_params import BudgetInfoParams as BudgetInfoParams
from .file_create_params import FileCreateParams as FileCreateParams
from .key_block_response import KeyBlockResponse as KeyBlockResponse
from .key_unblock_params import KeyUnblockParams as KeyUnblockParams
from .team_create_params import TeamCreateParams as TeamCreateParams
from .team_delete_params import TeamDeleteParams as TeamDeleteParams
from .team_update_params import TeamUpdateParams as TeamUpdateParams
from .user_create_params import UserCreateParams as UserCreateParams
from .user_delete_params import UserDeleteParams as UserDeleteParams
from .user_update_params import UserUpdateParams as UserUpdateParams
from .batch_create_params import BatchCreateParams as BatchCreateParams
from .cache_ping_response import CachePingResponse as CachePingResponse
from .key_generate_params import KeyGenerateParams as KeyGenerateParams
from .lite_llm_spend_logs import LiteLlmSpendLogs as LiteLlmSpendLogs
from .lite_llm_team_table import LiteLlmTeamTable as LiteLlmTeamTable
from .lite_llm_user_table import LiteLlmUserTable as LiteLlmUserTable
from .model_create_params import ModelCreateParams as ModelCreateParams
from .model_delete_params import ModelDeleteParams as ModelDeleteParams
from .team_unblock_params import TeamUnblockParams as TeamUnblockParams
from .budget_create_params import BudgetCreateParams as BudgetCreateParams
from .budget_delete_params import BudgetDeleteParams as BudgetDeleteParams
from .budget_update_params import BudgetUpdateParams as BudgetUpdateParams
from .lite_llm_model_table import LiteLlmModelTable as LiteLlmModelTable
from .user_create_response import UserCreateResponse as UserCreateResponse
from .batch_retrieve_params import BatchRetrieveParams as BatchRetrieveParams
from .customer_block_params import CustomerBlockParams as CustomerBlockParams
from .generate_key_response import GenerateKeyResponse as GenerateKeyResponse
from .budget_settings_params import BudgetSettingsParams as BudgetSettingsParams
from .customer_create_params import CustomerCreateParams as CustomerCreateParams
from .customer_delete_params import CustomerDeleteParams as CustomerDeleteParams
from .customer_list_response import CustomerListResponse as CustomerListResponse
from .customer_update_params import CustomerUpdateParams as CustomerUpdateParams
from .spend_list_logs_params import SpendListLogsParams as SpendListLogsParams
from .spend_list_tags_params import SpendListTagsParams as SpendListTagsParams
from .team_add_member_params import TeamAddMemberParams as TeamAddMemberParams
from .customer_unblock_params import CustomerUnblockParams as CustomerUnblockParams
from .embedding_create_params import EmbeddingCreateParams as EmbeddingCreateParams
from .guardrail_list_response import GuardrailListResponse as GuardrailListResponse
from .health_check_all_params import HealthCheckAllParams as HealthCheckAllParams
from .lite_llm_end_user_table import LiteLlmEndUserTable as LiteLlmEndUserTable
from .completion_create_params import CompletionCreateParams as CompletionCreateParams
from .credential_create_params import CredentialCreateParams as CredentialCreateParams
from .credential_update_params import CredentialUpdateParams as CredentialUpdateParams
from .key_retrieve_info_params import KeyRetrieveInfoParams as KeyRetrieveInfoParams
from .spend_list_logs_response import SpendListLogsResponse as SpendListLogsResponse
from .spend_list_tags_response import SpendListTagsResponse as SpendListTagsResponse
from .team_add_member_response import TeamAddMemberResponse as TeamAddMemberResponse
from .add_add_allowed_ip_params import AddAddAllowedIPParams as AddAddAllowedIPParams
from .key_check_health_response import KeyCheckHealthResponse as KeyCheckHealthResponse
from .team_remove_member_params import TeamRemoveMemberParams as TeamRemoveMemberParams
from .team_retrieve_info_params import TeamRetrieveInfoParams as TeamRetrieveInfoParams
from .team_update_member_params import TeamUpdateMemberParams as TeamUpdateMemberParams
from .user_retrieve_info_params import UserRetrieveInfoParams as UserRetrieveInfoParams
from .util_token_counter_params import UtilTokenCounterParams as UtilTokenCounterParams
from .organization_create_params import OrganizationCreateParams as OrganizationCreateParams
from .organization_delete_params import OrganizationDeleteParams as OrganizationDeleteParams
from .organization_list_response import OrganizationListResponse as OrganizationListResponse
from .organization_update_params import OrganizationUpdateParams as OrganizationUpdateParams
from .team_list_available_params import TeamListAvailableParams as TeamListAvailableParams
from .team_update_member_response import TeamUpdateMemberResponse as TeamUpdateMemberResponse
from .util_token_counter_response import UtilTokenCounterResponse as UtilTokenCounterResponse
from .health_check_services_params import HealthCheckServicesParams as HealthCheckServicesParams
from .key_regenerate_by_key_params import KeyRegenerateByKeyParams as KeyRegenerateByKeyParams
from .organization_create_response import OrganizationCreateResponse as OrganizationCreateResponse
from .organization_delete_response import OrganizationDeleteResponse as OrganizationDeleteResponse
from .spend_calculate_spend_params import SpendCalculateSpendParams as SpendCalculateSpendParams
from .customer_retrieve_info_params import CustomerRetrieveInfoParams as CustomerRetrieveInfoParams
from .organization_membership_table import OrganizationMembershipTable as OrganizationMembershipTable
from .util_transform_request_params import UtilTransformRequestParams as UtilTransformRequestParams
from .organization_add_member_params import OrganizationAddMemberParams as OrganizationAddMemberParams
from .provider_list_budgets_response import ProviderListBudgetsResponse as ProviderListBudgetsResponse
from .batch_list_with_provider_params import BatchListWithProviderParams as BatchListWithProviderParams
from .delete_create_allowed_ip_params import DeleteCreateAllowedIPParams as DeleteCreateAllowedIPParams
from .organization_table_with_members import OrganizationTableWithMembers as OrganizationTableWithMembers
from .util_transform_request_response import UtilTransformRequestResponse as UtilTransformRequestResponse
from .model_group_retrieve_info_params import ModelGroupRetrieveInfoParams as ModelGroupRetrieveInfoParams
from .organization_add_member_response import OrganizationAddMemberResponse as OrganizationAddMemberResponse
from .organization_delete_member_params import OrganizationDeleteMemberParams as OrganizationDeleteMemberParams
from .organization_update_member_params import OrganizationUpdateMemberParams as OrganizationUpdateMemberParams
from .util_get_supported_openai_params_params import (
    UtilGetSupportedOpenAIParamsParams as UtilGetSupportedOpenAIParamsParams,
)
from .configurable_clientside_params_custom_auth_param import (
    ConfigurableClientsideParamsCustomAuthParam as ConfigurableClientsideParamsCustomAuthParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .chat import (
    ChatResource,
    AsyncChatResource,
    ChatResourceWithRawResponse,
    AsyncChatResourceWithRawResponse,
    ChatResourceWithStreamingResponse,
    AsyncChatResourceWithStreamingResponse,
)
from .deployments import (
    DeploymentsResource,
    AsyncDeploymentsResource,
    DeploymentsResourceWithRawResponse,
    AsyncDeploymentsResourceWithRawResponse,
    DeploymentsResourceWithStreamingResponse,
    AsyncDeploymentsResourceWithStreamingResponse,
)

__all__ = [
    "ChatResource",
    "AsyncChatResource",
    "ChatResourceWithRawResponse",
    "AsyncChatResourceWithRawResponse",
    "ChatResourceWithStreamingResponse",
    "AsyncChatResourceWithStreamingResponse",
    "DeploymentsResource",
    "AsyncDeploymentsResource",
    "DeploymentsResourceWithRawResponse",
    "AsyncDeploymentsResourceWithRawResponse",
    "DeploymentsResourceWithStreamingResponse",
    "AsyncDeploymentsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PassThroughEndpointCreateParams"]


class PassThroughEndpointCreateParams(TypedDict, total=False):
    headers: Required[object]
    """Key-value pairs of headers to be forwarded with the request.

    You can set any key value pair here and it will be forwarded to your target
    endpoint
    """

    path: Required[str]
    """The route to be added to the LiteLLM Proxy Server."""

    target: Required[str]
    """The URL to which requests for this path should be forwarded."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hanzoai import Hanzo, AsyncHanzo
from tests.utils import assert_matches_type
from hanzoai.types import (
    LiteLlmEndUserTable,
    CustomerListResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCustomer:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: Hanzo) -> None:
        customer = client.customer.create(
            user_id="user_id",
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: Hanzo) -> None:
        customer = client.customer.create(
            user_id="user_id",
            alias="alias",
            allowed_model_region="eu",
            blocked=True,
            budget_duration="budget_duration",
            budget_id="budget_id",
            default_model="default_model",
            max_budget=0,
            max_parallel_requests=0,
            model_max_budget={
                "foo": {
                    "budget_duration": "budget_duration",
                    "max_budget": 0,
                    "rpm_limit": 0,
                    "tpm_limit": 0,
                }
            },
            rpm_limit=0,
            soft_budget=0,
            tpm_limit=0,
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: Hanzo) -> None:
        response = client.customer.with_raw_response.create(
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: Hanzo) -> None:
        with client.customer.with_streaming_response.create(
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(object, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: Hanzo) -> None:
        customer = client.customer.update(
            user_id="user_id",
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: Hanzo) -> None:
        customer = client.customer.update(
            user_id="user_id",
            alias="alias",
            allowed_model_region="eu",
            blocked=True,
            budget_id="budget_id",
            default_model="default_model",
            max_budget=0,
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: Hanzo) -> None:
        response = client.customer.with_raw_response.update(
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: Hanzo) -> None:
        with client.customer.with_streaming_response.update(
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(object, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: Hanzo) -> None:
        customer = client.customer.list()
        assert_matches_type(CustomerListResponse, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: Hanzo) -> None:
        response = client.customer.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(CustomerListResponse, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: Hanzo) -> None:
        with client.customer.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(CustomerListResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: Hanzo) -> None:
        customer = client.customer.delete(
            user_ids=["string"],
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: Hanzo) -> None:
        response = client.customer.with_raw_response.delete(
            user_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: Hanzo) -> None:
        with client.customer.with_streaming_response.delete(
            user_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(object, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_block(self, client: Hanzo) -> None:
        customer = client.customer.block(
            user_ids=["string"],
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_block(self, client: Hanzo) -> None:
        response = client.customer.with_raw_response.block(
            user_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_block(self, client: Hanzo) -> None:
        with client.customer.with_streaming_response.block(
            user_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(object, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_info(self, client: Hanzo) -> None:
        customer = client.customer.retrieve_info(
            end_user_id="end_user_id",
        )
        assert_matches_type(LiteLlmEndUserTable, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_info(self, client: Hanzo) -> None:
        response = client.customer.with_raw_response.retrieve_info(
            end_user_id="end_user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(LiteLlmEndUserTable, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_info(self, client: Hanzo) -> None:
        with client.customer.with_streaming_response.retrieve_info(
            end_user_id="end_user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(LiteLlmEndUserTable, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_unblock(self, client: Hanzo) -> None:
        customer = client.customer.unblock(
            user_ids=["string"],
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_unblock(self, client: Hanzo) -> None:
        response = client.customer.with_raw_response.unblock(
            user_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = response.parse()
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_unblock(self, client: Hanzo) -> None:
        with client.customer.with_streaming_response.unblock(
            user_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = response.parse()
            assert_matches_type(object, customer, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCustomer:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncHanzo) -> None:
        customer = await async_client.customer.create(
            user_id="user_id",
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncHanzo) -> None:
        customer = await async_client.customer.create(
            user_id="user_id",
            alias="alias",
            allowed_model_region="eu",
            blocked=True,
            budget_duration="budget_duration",
            budget_id="budget_id",
            default_model="default_model",
            max_budget=0,
            max_parallel_requests=0,
            model_max_budget={
                "foo": {
                    "budget_duration": "budget_duration",
                    "max_budget": 0,
                    "rpm_limit": 0,
                    "tpm_limit": 0,
                }
            },
            rpm_limit=0,
            soft_budget=0,
            tpm_limit=0,
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncHanzo) -> None:
        response = await async_client.customer.with_raw_response.create(
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncHanzo) -> None:
        async with async_client.customer.with_streaming_response.create(
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(object, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncHanzo) -> None:
        customer = await async_client.customer.update(
            user_id="user_id",
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHanzo) -> None:
        customer = await async_client.customer.update(
            user_id="user_id",
            alias="alias",
            allowed_model_region="eu",
            blocked=True,
            budget_id="budget_id",
            default_model="default_model",
            max_budget=0,
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHanzo) -> None:
        response = await async_client.customer.with_raw_response.update(
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHanzo) -> None:
        async with async_client.customer.with_streaming_response.update(
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(object, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncHanzo) -> None:
        customer = await async_client.customer.list()
        assert_matches_type(CustomerListResponse, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHanzo) -> None:
        response = await async_client.customer.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(CustomerListResponse, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHanzo) -> None:
        async with async_client.customer.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(CustomerListResponse, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncHanzo) -> None:
        customer = await async_client.customer.delete(
            user_ids=["string"],
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncHanzo) -> None:
        response = await async_client.customer.with_raw_response.delete(
            user_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncHanzo) -> None:
        async with async_client.customer.with_streaming_response.delete(
            user_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(object, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_block(self, async_client: AsyncHanzo) -> None:
        customer = await async_client.customer.block(
            user_ids=["string"],
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_block(self, async_client: AsyncHanzo) -> None:
        response = await async_client.customer.with_raw_response.block(
            user_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_block(self, async_client: AsyncHanzo) -> None:
        async with async_client.customer.with_streaming_response.block(
            user_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(object, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_info(self, async_client: AsyncHanzo) -> None:
        customer = await async_client.customer.retrieve_info(
            end_user_id="end_user_id",
        )
        assert_matches_type(LiteLlmEndUserTable, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_info(self, async_client: AsyncHanzo) -> None:
        response = await async_client.customer.with_raw_response.retrieve_info(
            end_user_id="end_user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(LiteLlmEndUserTable, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_info(self, async_client: AsyncHanzo) -> None:
        async with async_client.customer.with_streaming_response.retrieve_info(
            end_user_id="end_user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(LiteLlmEndUserTable, customer, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_unblock(self, async_client: AsyncHanzo) -> None:
        customer = await async_client.customer.unblock(
            user_ids=["string"],
        )
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_unblock(self, async_client: AsyncHanzo) -> None:
        response = await async_client.customer.with_raw_response.unblock(
            user_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        customer = await response.parse()
        assert_matches_type(object, customer, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_unblock(self, async_client: AsyncHanzo) -> None:
        async with async_client.customer.with_streaming_response.unblock(
            user_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            customer = await response.parse()
            assert_matches_type(object, customer, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["UtilTokenCounterResponse"]


class UtilTokenCounterResponse(BaseModel):
    api_model_used: str = FieldInfo(alias="model_used")

    request_model: str

    tokenizer_type: str

    total_tokens: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from typing_extensions import TypeAlias, TypedDict

from ..model_info_param import ModelInfoParam
from ..configurable_clientside_params_custom_auth_param import ConfigurableClientsideParamsCustomAuthParam

__all__ = ["UpdateFullParams", "LitellmParams", "LitellmParamsConfigurableClientsideAuthParam"]


class UpdateFullParams(TypedDict, total=False):
    litellm_params: Optional[LitellmParams]

    model_info: Optional[ModelInfoParam]

    model_name: Optional[str]


LitellmParamsConfigurableClientsideAuthParam: TypeAlias = Union[str, ConfigurableClientsideParamsCustomAuthParam]


class LitellmParamsTyped(TypedDict, total=False):
    api_base: Optional[str]

    api_key: Optional[str]

    api_version: Optional[str]

    aws_access_key_id: Optional[str]

    aws_region_name: Optional[str]

    aws_secret_access_key: Optional[str]

    budget_duration: Optional[str]

    configurable_clientside_auth_params: Optional[List[LitellmParamsConfigurableClientsideAuthParam]]

    custom_llm_provider: Optional[str]

    input_cost_per_second: Optional[float]

    input_cost_per_token: Optional[float]

    litellm_trace_id: Optional[str]

    max_budget: Optional[float]

    max_file_size_mb: Optional[float]

    max_retries: Optional[int]

    merge_reasoning_content_in_choices: Optional[bool]

    model: Optional[str]

    model_info: Optional[object]

    organization: Optional[str]

    output_cost_per_second: Optional[float]

    output_cost_per_token: Optional[float]

    region_name: Optional[str]

    rpm: Optional[int]

    stream_timeout: Union[float, str, None]

    timeout: Union[float, str, None]

    tpm: Optional[int]

    use_in_pass_through: Optional[bool]

    vertex_credentials: Union[str, object, None]

    vertex_location: Optional[str]

    vertex_project: Optional[str]

    watsonx_region_name: Optional[str]


LitellmParams: TypeAlias = Union[LitellmParamsTyped, Dict[str, object]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hanzoai import Hanzo, AsyncHanzo
from tests.utils import assert_matches_type
from hanzoai._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUpdate:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_full(self, client: Hanzo) -> None:
        update = client.model.update.full()
        assert_matches_type(object, update, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_full_with_all_params(self, client: Hanzo) -> None:
        update = client.model.update.full(
            litellm_params={
                "api_base": "api_base",
                "api_key": "api_key",
                "api_version": "api_version",
                "aws_access_key_id": "aws_access_key_id",
                "aws_region_name": "aws_region_name",
                "aws_secret_access_key": "aws_secret_access_key",
                "budget_duration": "budget_duration",
                "configurable_clientside_auth_params": ["string"],
                "custom_llm_provider": "custom_llm_provider",
                "input_cost_per_second": 0,
                "input_cost_per_token": 0,
                "litellm_trace_id": "litellm_trace_id",
                "max_budget": 0,
                "max_file_size_mb": 0,
                "max_retries": 0,
                "merge_reasoning_content_in_choices": True,
                "model": "model",
                "model_info": {},
                "organization": "organization",
                "output_cost_per_second": 0,
                "output_cost_per_token": 0,
                "region_name": "region_name",
                "rpm": 0,
                "stream_timeout": 0,
                "timeout": 0,
                "tpm": 0,
                "use_in_pass_through": True,
                "vertex_credentials": "string",
                "vertex_location": "vertex_location",
                "vertex_project": "vertex_project",
                "watsonx_region_name": "watsonx_region_name",
            },
            model_info={
                "id": "id",
                "base_model": "base_model",
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "created_by": "created_by",
                "db_model": True,
                "team_id": "team_id",
                "team_public_model_name": "team_public_model_name",
                "tier": "free",
                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "updated_by": "updated_by",
            },
            model_name="model_name",
        )
        assert_matches_type(object, update, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_full(self, client: Hanzo) -> None:
        response = client.model.update.with_raw_response.full()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        update = response.parse()
        assert_matches_type(object, update, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_full(self, client: Hanzo) -> None:
        with client.model.update.with_streaming_response.full() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            update = response.parse()
            assert_matches_type(object, update, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_partial(self, client: Hanzo) -> None:
        update = client.model.update.partial(
            model_id="model_id",
        )
        assert_matches_type(object, update, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_partial_with_all_params(self, client: Hanzo) -> None:
        update = client.model.update.partial(
            model_id="model_id",
            litellm_params={
                "api_base": "api_base",
                "api_key": "api_key",
                "api_version": "api_version",
                "aws_access_key_id": "aws_access_key_id",
                "aws_region_name": "aws_region_name",
                "aws_secret_access_key": "aws_secret_access_key",
                "budget_duration": "budget_duration",
                "configurable_clientside_auth_params": ["string"],
                "custom_llm_provider": "custom_llm_provider",
                "input_cost_per_second": 0,
                "input_cost_per_token": 0,
                "litellm_trace_id": "litellm_trace_id",
                "max_budget": 0,
                "max_file_size_mb": 0,
                "max_retries": 0,
                "merge_reasoning_content_in_choices": True,
                "model": "model",
                "model_info": {},
                "organization": "organization",
                "output_cost_per_second": 0,
                "output_cost_per_token": 0,
                "region_name": "region_name",
                "rpm": 0,
                "stream_timeout": 0,
                "timeout": 0,
                "tpm": 0,
                "use_in_pass_through": True,
                "vertex_credentials": "string",
                "vertex_location": "vertex_location",
                "vertex_project": "vertex_project",
                "watsonx_region_name": "watsonx_region_name",
            },
            model_info={
                "id": "id",
                "base_model": "base_model",
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "created_by": "created_by",
                "db_model": True,
                "team_id": "team_id",
                "team_public_model_name": "team_public_model_name",
                "tier": "free",
                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "updated_by": "updated_by",
            },
            model_name="model_name",
        )
        assert_matches_type(object, update, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_partial(self, client: Hanzo) -> None:
        response = client.model.update.with_raw_response.partial(
            model_id="model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        update = response.parse()
        assert_matches_type(object, update, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_partial(self, client: Hanzo) -> None:
        with client.model.update.with_streaming_response.partial(
            model_id="model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            update = response.parse()
            assert_matches_type(object, update, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_partial(self, client: Hanzo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.model.update.with_raw_response.partial(
                model_id="",
            )


class TestAsyncUpdate:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_full(self, async_client: AsyncHanzo) -> None:
        update = await async_client.model.update.full()
        assert_matches_type(object, update, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_full_with_all_params(self, async_client: AsyncHanzo) -> None:
        update = await async_client.model.update.full(
            litellm_params={
                "api_base": "api_base",
                "api_key": "api_key",
                "api_version": "api_version",
                "aws_access_key_id": "aws_access_key_id",
                "aws_region_name": "aws_region_name",
                "aws_secret_access_key": "aws_secret_access_key",
                "budget_duration": "budget_duration",
                "configurable_clientside_auth_params": ["string"],
                "custom_llm_provider": "custom_llm_provider",
                "input_cost_per_second": 0,
                "input_cost_per_token": 0,
                "litellm_trace_id": "litellm_trace_id",
                "max_budget": 0,
                "max_file_size_mb": 0,
                "max_retries": 0,
                "merge_reasoning_content_in_choices": True,
                "model": "model",
                "model_info": {},
                "organization": "organization",
                "output_cost_per_second": 0,
                "output_cost_per_token": 0,
                "region_name": "region_name",
                "rpm": 0,
                "stream_timeout": 0,
                "timeout": 0,
                "tpm": 0,
                "use_in_pass_through": True,
                "vertex_credentials": "string",
                "vertex_location": "vertex_location",
                "vertex_project": "vertex_project",
                "watsonx_region_name": "watsonx_region_name",
            },
            model_info={
                "id": "id",
                "base_model": "base_model",
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "created_by": "created_by",
                "db_model": True,
                "team_id": "team_id",
                "team_public_model_name": "team_public_model_name",
                "tier": "free",
                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "updated_by": "updated_by",
            },
            model_name="model_name",
        )
        assert_matches_type(object, update, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_full(self, async_client: AsyncHanzo) -> None:
        response = await async_client.model.update.with_raw_response.full()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        update = await response.parse()
        assert_matches_type(object, update, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_full(self, async_client: AsyncHanzo) -> None:
        async with async_client.model.update.with_streaming_response.full() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            update = await response.parse()
            assert_matches_type(object, update, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_partial(self, async_client: AsyncHanzo) -> None:
        update = await async_client.model.update.partial(
            model_id="model_id",
        )
        assert_matches_type(object, update, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_partial_with_all_params(self, async_client: AsyncHanzo) -> None:
        update = await async_client.model.update.partial(
            model_id="model_id",
            litellm_params={
                "api_base": "api_base",
                "api_key": "api_key",
                "api_version": "api_version",
                "aws_access_key_id": "aws_access_key_id",
                "aws_region_name": "aws_region_name",
                "aws_secret_access_key": "aws_secret_access_key",
                "budget_duration": "budget_duration",
                "configurable_clientside_auth_params": ["string"],
                "custom_llm_provider": "custom_llm_provider",
                "input_cost_per_second": 0,
                "input_cost_per_token": 0,
                "litellm_trace_id": "litellm_trace_id",
                "max_budget": 0,
                "max_file_size_mb": 0,
                "max_retries": 0,
                "merge_reasoning_content_in_choices": True,
                "model": "model",
                "model_info": {},
                "organization": "organization",
                "output_cost_per_second": 0,
                "output_cost_per_token": 0,
                "region_name": "region_name",
                "rpm": 0,
                "stream_timeout": 0,
                "timeout": 0,
                "tpm": 0,
                "use_in_pass_through": True,
                "vertex_credentials": "string",
                "vertex_location": "vertex_location",
                "vertex_project": "vertex_project",
                "watsonx_region_name": "watsonx_region_name",
            },
            model_info={
                "id": "id",
                "base_model": "base_model",
                "created_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "created_by": "created_by",
                "db_model": True,
                "team_id": "team_id",
                "team_public_model_name": "team_public_model_name",
                "tier": "free",
                "updated_at": parse_datetime("2019-12-27T18:11:19.117Z"),
                "updated_by": "updated_by",
            },
            model_name="model_name",
        )
        assert_matches_type(object, update, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_partial(self, async_client: AsyncHanzo) -> None:
        response = await async_client.model.update.with_raw_response.partial(
            model_id="model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        update = await response.parse()
        assert_matches_type(object, update, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_partial(self, async_client: AsyncHanzo) -> None:
        async with async_client.model.update.with_streaming_response.partial(
            model_id="model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            update = await response.parse()
            assert_matches_type(object, update, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_partial(self, async_client: AsyncHanzo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.model.update.with_raw_response.partial(
                model_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.global_ import spend_list_tags_params, spend_retrieve_report_params
from ...types.global_.spend_list_tags_response import SpendListTagsResponse
from ...types.global_.spend_retrieve_report_response import SpendRetrieveReportResponse

__all__ = ["SpendResource", "AsyncSpendResource"]


class SpendResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpendResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return SpendResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpendResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return SpendResourceWithStreamingResponse(self)

    def list_tags(
        self,
        *,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        tags: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SpendListTagsResponse:
        """LiteLLM Enterprise - View Spend Per Request Tag.

        Used by LiteLLM UI

        Example Request:

        ```
        curl -X GET "http://0.0.0.0:4000/spend/tags" -H "Authorization: Bearer sk-1234"
        ```

        Spend with Start Date and End Date

        ```
        curl -X GET "http://0.0.0.0:4000/spend/tags?start_date=2022-01-01&end_date=2022-02-01" -H "Authorization: Bearer sk-1234"
        ```

        Args:
          end_date: Time till which to view key spend

          start_date: Time from which to start viewing key spend

          tags: comman separated tags to filter on

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/global/spend/tags",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_date": end_date,
                        "start_date": start_date,
                        "tags": tags,
                    },
                    spend_list_tags_params.SpendListTagsParams,
                ),
            ),
            cast_to=SpendListTagsResponse,
        )

    def reset(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        ADMIN ONLY / MASTER KEY Only Endpoint

        Globally reset spend for All API Keys and Teams, maintain LiteLLM_SpendLogs

        1. LiteLLM_SpendLogs will maintain the logs on spend, no data gets deleted from
           there
        2. LiteLLM_VerificationTokens spend will be set = 0
        3. LiteLLM_TeamTable spend will be set = 0
        """
        return self._post(
            "/global/spend/reset",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def retrieve_report(
        self,
        *,
        api_key: Optional[str] | NotGiven = NOT_GIVEN,
        customer_id: Optional[str] | NotGiven = NOT_GIVEN,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        group_by: Optional[Literal["team", "customer", "api_key"]] | NotGiven = NOT_GIVEN,
        internal_user_id: Optional[str] | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        team_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SpendRetrieveReportResponse:
        """Get Daily Spend per Team, based on specific startTime and endTime.

        Per team,
        view usage by each key, model [ { "group-by-day": "2024-05-10", "teams": [ {
        "team_name": "team-1" "spend": 10, "keys": [ "key": "1213", "usage": {
        "model-1": { "cost": 12.50, "input_tokens": 1000, "output_tokens": 5000,
        "requests": 100 }, "audio-modelname1": { "cost": 25.50, "seconds": 25,
        "requests": 50 }, } } ] ] }

        Args:
          api_key: View spend for a specific api_key. Example api_key='sk-1234

          customer_id: View spend for a specific customer_id. Example customer_id='1234. Can be used in
              conjunction with team_id as well.

          end_date: Time till which to view spend

          group_by: Group spend by internal team or customer or api_key

          internal_user_id: View spend for a specific internal_user_id. Example internal_user_id='1234

          start_date: Time from which to start viewing spend

          team_id: View spend for a specific team_id. Example team_id='1234

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/global/spend/report",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "api_key": api_key,
                        "customer_id": customer_id,
                        "end_date": end_date,
                        "group_by": group_by,
                        "internal_user_id": internal_user_id,
                        "start_date": start_date,
                        "team_id": team_id,
                    },
                    spend_retrieve_report_params.SpendRetrieveReportParams,
                ),
            ),
            cast_to=SpendRetrieveReportResponse,
        )


class AsyncSpendResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSpendResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncSpendResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpendResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncSpendResourceWithStreamingResponse(self)

    async def list_tags(
        self,
        *,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        tags: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SpendListTagsResponse:
        """LiteLLM Enterprise - View Spend Per Request Tag.

        Used by LiteLLM UI

        Example Request:

        ```
        curl -X GET "http://0.0.0.0:4000/spend/tags" -H "Authorization: Bearer sk-1234"
        ```

        Spend with Start Date and End Date

        ```
        curl -X GET "http://0.0.0.0:4000/spend/tags?start_date=2022-01-01&end_date=2022-02-01" -H "Authorization: Bearer sk-1234"
        ```

        Args:
          end_date: Time till which to view key spend

          start_date: Time from which to start viewing key spend

          tags: comman separated tags to filter on

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/global/spend/tags",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "end_date": end_date,
                        "start_date": start_date,
                        "tags": tags,
                    },
                    spend_list_tags_params.SpendListTagsParams,
                ),
            ),
            cast_to=SpendListTagsResponse,
        )

    async def reset(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        ADMIN ONLY / MASTER KEY Only Endpoint

        Globally reset spend for All API Keys and Teams, maintain LiteLLM_SpendLogs

        1. LiteLLM_SpendLogs will maintain the logs on spend, no data gets deleted from
           there
        2. LiteLLM_VerificationTokens spend will be set = 0
        3. LiteLLM_TeamTable spend will be set = 0
        """
        return await self._post(
            "/global/spend/reset",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def retrieve_report(
        self,
        *,
        api_key: Optional[str] | NotGiven = NOT_GIVEN,
        customer_id: Optional[str] | NotGiven = NOT_GIVEN,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        group_by: Optional[Literal["team", "customer", "api_key"]] | NotGiven = NOT_GIVEN,
        internal_user_id: Optional[str] | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        team_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SpendRetrieveReportResponse:
        """Get Daily Spend per Team, based on specific startTime and endTime.

        Per team,
        view usage by each key, model [ { "group-by-day": "2024-05-10", "teams": [ {
        "team_name": "team-1" "spend": 10, "keys": [ "key": "1213", "usage": {
        "model-1": { "cost": 12.50, "input_tokens": 1000, "output_tokens": 5000,
        "requests": 100 }, "audio-modelname1": { "cost": 25.50, "seconds": 25,
        "requests": 50 }, } } ] ] }

        Args:
          api_key: View spend for a specific api_key. Example api_key='sk-1234

          customer_id: View spend for a specific customer_id. Example customer_id='1234. Can be used in
              conjunction with team_id as well.

          end_date: Time till which to view spend

          group_by: Group spend by internal team or customer or api_key

          internal_user_id: View spend for a specific internal_user_id. Example internal_user_id='1234

          start_date: Time from which to start viewing spend

          team_id: View spend for a specific team_id. Example team_id='1234

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/global/spend/report",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "api_key": api_key,
                        "customer_id": customer_id,
                        "end_date": end_date,
                        "group_by": group_by,
                        "internal_user_id": internal_user_id,
                        "start_date": start_date,
                        "team_id": team_id,
                    },
                    spend_retrieve_report_params.SpendRetrieveReportParams,
                ),
            ),
            cast_to=SpendRetrieveReportResponse,
        )


class SpendResourceWithRawResponse:
    def __init__(self, spend: SpendResource) -> None:
        self._spend = spend

        self.list_tags = to_raw_response_wrapper(
            spend.list_tags,
        )
        self.reset = to_raw_response_wrapper(
            spend.reset,
        )
        self.retrieve_report = to_raw_response_wrapper(
            spend.retrieve_report,
        )


class AsyncSpendResourceWithRawResponse:
    def __init__(self, spend: AsyncSpendResource) -> None:
        self._spend = spend

        self.list_tags = async_to_raw_response_wrapper(
            spend.list_tags,
        )
        self.reset = async_to_raw_response_wrapper(
            spend.reset,
        )
        self.retrieve_report = async_to_raw_response_wrapper(
            spend.retrieve_report,
        )


class SpendResourceWithStreamingResponse:
    def __init__(self, spend: SpendResource) -> None:
        self._spend = spend

        self.list_tags = to_streamed_response_wrapper(
            spend.list_tags,
        )
        self.reset = to_streamed_response_wrapper(
            spend.reset,
        )
        self.retrieve_report = to_streamed_response_wrapper(
            spend.retrieve_report,
        )


class AsyncSpendResourceWithStreamingResponse:
    def __init__(self, spend: AsyncSpendResource) -> None:
        self._spend = spend

        self.list_tags = async_to_streamed_response_wrapper(
            spend.list_tags,
        )
        self.reset = async_to_streamed_response_wrapper(
            spend.reset,
        )
        self.retrieve_report = async_to_streamed_response_wrapper(
            spend.retrieve_report,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .chat import (
    ChatResource,
    AsyncChatResource,
    ChatResourceWithRawResponse,
    AsyncChatResourceWithRawResponse,
    ChatResourceWithStreamingResponse,
    AsyncChatResourceWithStreamingResponse,
)
from .engines import (
    EnginesResource,
    AsyncEnginesResource,
    EnginesResourceWithRawResponse,
    AsyncEnginesResourceWithRawResponse,
    EnginesResourceWithStreamingResponse,
    AsyncEnginesResourceWithStreamingResponse,
)

__all__ = [
    "ChatResource",
    "AsyncChatResource",
    "ChatResourceWithRawResponse",
    "AsyncChatResourceWithRawResponse",
    "ChatResourceWithStreamingResponse",
    "AsyncChatResourceWithStreamingResponse",
    "EnginesResource",
    "AsyncEnginesResource",
    "EnginesResourceWithRawResponse",
    "AsyncEnginesResourceWithRawResponse",
    "EnginesResourceWithStreamingResponse",
    "AsyncEnginesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .info_retrieve_params import InfoRetrieveParams as InfoRetrieveParams
from .info_deprecated_params import InfoDeprecatedParams as InfoDeprecatedParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Required, TypeAlias, TypedDict

from .member_param import MemberParam

__all__ = ["TeamAddMemberParams", "Member"]


class TeamAddMemberParams(TypedDict, total=False):
    member: Required[Member]

    team_id: Required[str]

    max_budget_in_team: Optional[float]


Member: TypeAlias = Union[Iterable[MemberParam], MemberParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .spend import (
    SpendResource,
    AsyncSpendResource,
    SpendResourceWithRawResponse,
    AsyncSpendResourceWithRawResponse,
    SpendResourceWithStreamingResponse,
    AsyncSpendResourceWithStreamingResponse,
)
from .global_ import (
    GlobalResource,
    AsyncGlobalResource,
    GlobalResourceWithRawResponse,
    AsyncGlobalResourceWithRawResponse,
    GlobalResourceWithStreamingResponse,
    AsyncGlobalResourceWithStreamingResponse,
)

__all__ = [
    "SpendResource",
    "AsyncSpendResource",
    "SpendResourceWithRawResponse",
    "AsyncSpendResourceWithRawResponse",
    "SpendResourceWithStreamingResponse",
    "AsyncSpendResourceWithStreamingResponse",
    "GlobalResource",
    "AsyncGlobalResource",
    "GlobalResourceWithRawResponse",
    "AsyncGlobalResourceWithRawResponse",
    "GlobalResourceWithStreamingResponse",
    "AsyncGlobalResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Optional, cast

import pytest

from hanzoai import Hanzo, AsyncHanzo
from tests.utils import assert_matches_type
from hanzoai.types import (
    KeyListResponse,
    KeyBlockResponse,
    GenerateKeyResponse,
    KeyCheckHealthResponse,
)
from hanzoai._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestKey:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: Hanzo) -> None:
        key = client.key.update(
            key="key",
        )
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: Hanzo) -> None:
        key = client.key.update(
            key="key",
            aliases={},
            allowed_cache_controls=[{}],
            blocked=True,
            budget_duration="budget_duration",
            budget_id="budget_id",
            config={},
            duration="duration",
            enforced_params=["string"],
            guardrails=["string"],
            key_alias="key_alias",
            max_budget=0,
            max_parallel_requests=0,
            metadata={},
            model_max_budget={},
            model_rpm_limit={},
            model_tpm_limit={},
            models=[{}],
            permissions={},
            rpm_limit=0,
            spend=0,
            tags=["string"],
            team_id="team_id",
            temp_budget_expiry=parse_datetime("2019-12-27T18:11:19.117Z"),
            temp_budget_increase=0,
            tpm_limit=0,
            user_id="user_id",
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: Hanzo) -> None:
        response = client.key.with_raw_response.update(
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = response.parse()
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: Hanzo) -> None:
        with client.key.with_streaming_response.update(
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = response.parse()
            assert_matches_type(object, key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: Hanzo) -> None:
        key = client.key.list()
        assert_matches_type(KeyListResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: Hanzo) -> None:
        key = client.key.list(
            include_team_keys=True,
            key_alias="key_alias",
            organization_id="organization_id",
            page=1,
            return_full_object=True,
            size=1,
            team_id="team_id",
            user_id="user_id",
        )
        assert_matches_type(KeyListResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: Hanzo) -> None:
        response = client.key.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = response.parse()
        assert_matches_type(KeyListResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: Hanzo) -> None:
        with client.key.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = response.parse()
            assert_matches_type(KeyListResponse, key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: Hanzo) -> None:
        key = client.key.delete()
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_delete_with_all_params(self, client: Hanzo) -> None:
        key = client.key.delete(
            key_aliases=["string"],
            keys=["string"],
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: Hanzo) -> None:
        response = client.key.with_raw_response.delete()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = response.parse()
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: Hanzo) -> None:
        with client.key.with_streaming_response.delete() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = response.parse()
            assert_matches_type(object, key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_block(self, client: Hanzo) -> None:
        key = client.key.block(
            key="key",
        )
        assert_matches_type(Optional[KeyBlockResponse], key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_block_with_all_params(self, client: Hanzo) -> None:
        key = client.key.block(
            key="key",
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(Optional[KeyBlockResponse], key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_block(self, client: Hanzo) -> None:
        response = client.key.with_raw_response.block(
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = response.parse()
        assert_matches_type(Optional[KeyBlockResponse], key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_block(self, client: Hanzo) -> None:
        with client.key.with_streaming_response.block(
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = response.parse()
            assert_matches_type(Optional[KeyBlockResponse], key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_check_health(self, client: Hanzo) -> None:
        key = client.key.check_health()
        assert_matches_type(KeyCheckHealthResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_check_health(self, client: Hanzo) -> None:
        response = client.key.with_raw_response.check_health()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = response.parse()
        assert_matches_type(KeyCheckHealthResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_check_health(self, client: Hanzo) -> None:
        with client.key.with_streaming_response.check_health() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = response.parse()
            assert_matches_type(KeyCheckHealthResponse, key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_generate(self, client: Hanzo) -> None:
        key = client.key.generate()
        assert_matches_type(GenerateKeyResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_generate_with_all_params(self, client: Hanzo) -> None:
        key = client.key.generate(
            aliases={},
            allowed_cache_controls=[{}],
            blocked=True,
            budget_duration="budget_duration",
            budget_id="budget_id",
            config={},
            duration="duration",
            enforced_params=["string"],
            guardrails=["string"],
            key="key",
            key_alias="key_alias",
            max_budget=0,
            max_parallel_requests=0,
            metadata={},
            model_max_budget={},
            model_rpm_limit={},
            model_tpm_limit={},
            models=[{}],
            permissions={},
            rpm_limit=0,
            send_invite_email=True,
            soft_budget=0,
            spend=0,
            tags=["string"],
            team_id="team_id",
            tpm_limit=0,
            user_id="user_id",
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(GenerateKeyResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_generate(self, client: Hanzo) -> None:
        response = client.key.with_raw_response.generate()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = response.parse()
        assert_matches_type(GenerateKeyResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_generate(self, client: Hanzo) -> None:
        with client.key.with_streaming_response.generate() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = response.parse()
            assert_matches_type(GenerateKeyResponse, key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_regenerate_by_key(self, client: Hanzo) -> None:
        key = client.key.regenerate_by_key(
            path_key="key",
        )
        assert_matches_type(Optional[GenerateKeyResponse], key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_regenerate_by_key_with_all_params(self, client: Hanzo) -> None:
        key = client.key.regenerate_by_key(
            path_key="key",
            aliases={},
            allowed_cache_controls=[{}],
            blocked=True,
            budget_duration="budget_duration",
            budget_id="budget_id",
            config={},
            duration="duration",
            enforced_params=["string"],
            guardrails=["string"],
            body_key="key",
            key_alias="key_alias",
            max_budget=0,
            max_parallel_requests=0,
            metadata={},
            model_max_budget={},
            model_rpm_limit={},
            model_tpm_limit={},
            models=[{}],
            new_master_key="new_master_key",
            permissions={},
            rpm_limit=0,
            send_invite_email=True,
            soft_budget=0,
            spend=0,
            tags=["string"],
            team_id="team_id",
            tpm_limit=0,
            user_id="user_id",
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(Optional[GenerateKeyResponse], key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_regenerate_by_key(self, client: Hanzo) -> None:
        response = client.key.with_raw_response.regenerate_by_key(
            path_key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = response.parse()
        assert_matches_type(Optional[GenerateKeyResponse], key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_regenerate_by_key(self, client: Hanzo) -> None:
        with client.key.with_streaming_response.regenerate_by_key(
            path_key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = response.parse()
            assert_matches_type(Optional[GenerateKeyResponse], key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_regenerate_by_key(self, client: Hanzo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `path_key` but received ''"):
            client.key.with_raw_response.regenerate_by_key(
                path_key="",
                body_key="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_info(self, client: Hanzo) -> None:
        key = client.key.retrieve_info()
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_info_with_all_params(self, client: Hanzo) -> None:
        key = client.key.retrieve_info(
            key="key",
        )
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_info(self, client: Hanzo) -> None:
        response = client.key.with_raw_response.retrieve_info()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = response.parse()
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_info(self, client: Hanzo) -> None:
        with client.key.with_streaming_response.retrieve_info() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = response.parse()
            assert_matches_type(object, key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_unblock(self, client: Hanzo) -> None:
        key = client.key.unblock(
            key="key",
        )
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_unblock_with_all_params(self, client: Hanzo) -> None:
        key = client.key.unblock(
            key="key",
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_unblock(self, client: Hanzo) -> None:
        response = client.key.with_raw_response.unblock(
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = response.parse()
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_unblock(self, client: Hanzo) -> None:
        with client.key.with_streaming_response.unblock(
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = response.parse()
            assert_matches_type(object, key, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncKey:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.update(
            key="key",
        )
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.update(
            key="key",
            aliases={},
            allowed_cache_controls=[{}],
            blocked=True,
            budget_duration="budget_duration",
            budget_id="budget_id",
            config={},
            duration="duration",
            enforced_params=["string"],
            guardrails=["string"],
            key_alias="key_alias",
            max_budget=0,
            max_parallel_requests=0,
            metadata={},
            model_max_budget={},
            model_rpm_limit={},
            model_tpm_limit={},
            models=[{}],
            permissions={},
            rpm_limit=0,
            spend=0,
            tags=["string"],
            team_id="team_id",
            temp_budget_expiry=parse_datetime("2019-12-27T18:11:19.117Z"),
            temp_budget_increase=0,
            tpm_limit=0,
            user_id="user_id",
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHanzo) -> None:
        response = await async_client.key.with_raw_response.update(
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = await response.parse()
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHanzo) -> None:
        async with async_client.key.with_streaming_response.update(
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = await response.parse()
            assert_matches_type(object, key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.list()
        assert_matches_type(KeyListResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.list(
            include_team_keys=True,
            key_alias="key_alias",
            organization_id="organization_id",
            page=1,
            return_full_object=True,
            size=1,
            team_id="team_id",
            user_id="user_id",
        )
        assert_matches_type(KeyListResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHanzo) -> None:
        response = await async_client.key.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = await response.parse()
        assert_matches_type(KeyListResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHanzo) -> None:
        async with async_client.key.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = await response.parse()
            assert_matches_type(KeyListResponse, key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.delete()
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.delete(
            key_aliases=["string"],
            keys=["string"],
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncHanzo) -> None:
        response = await async_client.key.with_raw_response.delete()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = await response.parse()
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncHanzo) -> None:
        async with async_client.key.with_streaming_response.delete() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = await response.parse()
            assert_matches_type(object, key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_block(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.block(
            key="key",
        )
        assert_matches_type(Optional[KeyBlockResponse], key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_block_with_all_params(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.block(
            key="key",
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(Optional[KeyBlockResponse], key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_block(self, async_client: AsyncHanzo) -> None:
        response = await async_client.key.with_raw_response.block(
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = await response.parse()
        assert_matches_type(Optional[KeyBlockResponse], key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_block(self, async_client: AsyncHanzo) -> None:
        async with async_client.key.with_streaming_response.block(
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = await response.parse()
            assert_matches_type(Optional[KeyBlockResponse], key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_check_health(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.check_health()
        assert_matches_type(KeyCheckHealthResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_check_health(self, async_client: AsyncHanzo) -> None:
        response = await async_client.key.with_raw_response.check_health()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = await response.parse()
        assert_matches_type(KeyCheckHealthResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_check_health(self, async_client: AsyncHanzo) -> None:
        async with async_client.key.with_streaming_response.check_health() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = await response.parse()
            assert_matches_type(KeyCheckHealthResponse, key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_generate(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.generate()
        assert_matches_type(GenerateKeyResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_generate_with_all_params(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.generate(
            aliases={},
            allowed_cache_controls=[{}],
            blocked=True,
            budget_duration="budget_duration",
            budget_id="budget_id",
            config={},
            duration="duration",
            enforced_params=["string"],
            guardrails=["string"],
            key="key",
            key_alias="key_alias",
            max_budget=0,
            max_parallel_requests=0,
            metadata={},
            model_max_budget={},
            model_rpm_limit={},
            model_tpm_limit={},
            models=[{}],
            permissions={},
            rpm_limit=0,
            send_invite_email=True,
            soft_budget=0,
            spend=0,
            tags=["string"],
            team_id="team_id",
            tpm_limit=0,
            user_id="user_id",
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(GenerateKeyResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_generate(self, async_client: AsyncHanzo) -> None:
        response = await async_client.key.with_raw_response.generate()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = await response.parse()
        assert_matches_type(GenerateKeyResponse, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_generate(self, async_client: AsyncHanzo) -> None:
        async with async_client.key.with_streaming_response.generate() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = await response.parse()
            assert_matches_type(GenerateKeyResponse, key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_regenerate_by_key(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.regenerate_by_key(
            path_key="key",
        )
        assert_matches_type(Optional[GenerateKeyResponse], key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_regenerate_by_key_with_all_params(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.regenerate_by_key(
            path_key="key",
            aliases={},
            allowed_cache_controls=[{}],
            blocked=True,
            budget_duration="budget_duration",
            budget_id="budget_id",
            config={},
            duration="duration",
            enforced_params=["string"],
            guardrails=["string"],
            body_key="key",
            key_alias="key_alias",
            max_budget=0,
            max_parallel_requests=0,
            metadata={},
            model_max_budget={},
            model_rpm_limit={},
            model_tpm_limit={},
            models=[{}],
            new_master_key="new_master_key",
            permissions={},
            rpm_limit=0,
            send_invite_email=True,
            soft_budget=0,
            spend=0,
            tags=["string"],
            team_id="team_id",
            tpm_limit=0,
            user_id="user_id",
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(Optional[GenerateKeyResponse], key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_regenerate_by_key(self, async_client: AsyncHanzo) -> None:
        response = await async_client.key.with_raw_response.regenerate_by_key(
            path_key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = await response.parse()
        assert_matches_type(Optional[GenerateKeyResponse], key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_regenerate_by_key(self, async_client: AsyncHanzo) -> None:
        async with async_client.key.with_streaming_response.regenerate_by_key(
            path_key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = await response.parse()
            assert_matches_type(Optional[GenerateKeyResponse], key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_regenerate_by_key(self, async_client: AsyncHanzo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `path_key` but received ''"):
            await async_client.key.with_raw_response.regenerate_by_key(
                path_key="",
                body_key="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_info(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.retrieve_info()
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_info_with_all_params(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.retrieve_info(
            key="key",
        )
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_info(self, async_client: AsyncHanzo) -> None:
        response = await async_client.key.with_raw_response.retrieve_info()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = await response.parse()
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_info(self, async_client: AsyncHanzo) -> None:
        async with async_client.key.with_streaming_response.retrieve_info() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = await response.parse()
            assert_matches_type(object, key, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_unblock(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.unblock(
            key="key",
        )
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_unblock_with_all_params(self, async_client: AsyncHanzo) -> None:
        key = await async_client.key.unblock(
            key="key",
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_unblock(self, async_client: AsyncHanzo) -> None:
        response = await async_client.key.with_raw_response.unblock(
            key="key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        key = await response.parse()
        assert_matches_type(object, key, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_unblock(self, async_client: AsyncHanzo) -> None:
        async with async_client.key.with_streaming_response.unblock(
            key="key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            key = await response.parse()
            assert_matches_type(object, key, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["CachePingResponse"]


class CachePingResponse(BaseModel):
    cache_type: str

    status: str

    health_check_cache_params: Optional[object] = None

    litellm_cache_params: Optional[str] = None

    ping_response: Optional[bool] = None

    set_cache_response: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable, Optional
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["KeyRegenerateByKeyParams"]


class KeyRegenerateByKeyParams(TypedDict, total=False):
    aliases: Optional[object]

    allowed_cache_controls: Optional[Iterable[object]]

    blocked: Optional[bool]

    budget_duration: Optional[str]

    budget_id: Optional[str]

    config: Optional[object]

    duration: Optional[str]

    enforced_params: Optional[List[str]]

    guardrails: Optional[List[str]]

    body_key: Annotated[Optional[str], PropertyInfo(alias="key")]

    key_alias: Optional[str]

    max_budget: Optional[float]

    max_parallel_requests: Optional[int]

    metadata: Optional[object]

    model_max_budget: Optional[object]

    model_rpm_limit: Optional[object]

    model_tpm_limit: Optional[object]

    models: Optional[Iterable[object]]

    new_master_key: Optional[str]

    permissions: Optional[object]

    rpm_limit: Optional[int]

    send_invite_email: Optional[bool]

    soft_budget: Optional[float]

    spend: Optional[float]

    tags: Optional[List[str]]

    team_id: Optional[str]

    tpm_limit: Optional[int]

    user_id: Optional[str]

    litellm_changed_by: Annotated[str, PropertyInfo(alias="litellm-changed-by")]
    """
    The litellm-changed-by header enables tracking of actions performed by
    authorized users on behalf of other users, providing an audit trail for
    accountability
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .info import (
    InfoResource,
    AsyncInfoResource,
    InfoResourceWithRawResponse,
    AsyncInfoResourceWithRawResponse,
    InfoResourceWithStreamingResponse,
    AsyncInfoResourceWithStreamingResponse,
)
from .update import (
    UpdateResource,
    AsyncUpdateResource,
    UpdateResourceWithRawResponse,
    AsyncUpdateResourceWithRawResponse,
    UpdateResourceWithStreamingResponse,
    AsyncUpdateResourceWithStreamingResponse,
)
from ...types import model_create_params, model_delete_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.model_info_param import ModelInfoParam

__all__ = ["ModelResource", "AsyncModelResource"]


class ModelResource(SyncAPIResource):
    @cached_property
    def info(self) -> InfoResource:
        return InfoResource(self._client)

    @cached_property
    def update(self) -> UpdateResource:
        return UpdateResource(self._client)

    @cached_property
    def with_raw_response(self) -> ModelResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return ModelResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ModelResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return ModelResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        litellm_params: model_create_params.LitellmParams,
        model_info: ModelInfoParam,
        model_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Allows adding new models to the model list in the config.yaml

        Args:
          litellm_params: LiteLLM Params with 'model' requirement - used for completions

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/model/new",
            body=maybe_transform(
                {
                    "litellm_params": litellm_params,
                    "model_info": model_info,
                    "model_name": model_name,
                },
                model_create_params.ModelCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def delete(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Allows deleting models in the model list in the config.yaml

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/model/delete",
            body=maybe_transform({"id": id}, model_delete_params.ModelDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncModelResource(AsyncAPIResource):
    @cached_property
    def info(self) -> AsyncInfoResource:
        return AsyncInfoResource(self._client)

    @cached_property
    def update(self) -> AsyncUpdateResource:
        return AsyncUpdateResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncModelResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hanzoai/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncModelResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncModelResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hanzoai/python-sdk#with_streaming_response
        """
        return AsyncModelResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        litellm_params: model_create_params.LitellmParams,
        model_info: ModelInfoParam,
        model_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Allows adding new models to the model list in the config.yaml

        Args:
          litellm_params: LiteLLM Params with 'model' requirement - used for completions

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/model/new",
            body=await async_maybe_transform(
                {
                    "litellm_params": litellm_params,
                    "model_info": model_info,
                    "model_name": model_name,
                },
                model_create_params.ModelCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def delete(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Allows deleting models in the model list in the config.yaml

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/model/delete",
            body=await async_maybe_transform({"id": id}, model_delete_params.ModelDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class ModelResourceWithRawResponse:
    def __init__(self, model: ModelResource) -> None:
        self._model = model

        self.create = to_raw_response_wrapper(
            model.create,
        )
        self.delete = to_raw_response_wrapper(
            model.delete,
        )

    @cached_property
    def info(self) -> InfoResourceWithRawResponse:
        return InfoResourceWithRawResponse(self._model.info)

    @cached_property
    def update(self) -> UpdateResourceWithRawResponse:
        return UpdateResourceWithRawResponse(self._model.update)


class AsyncModelResourceWithRawResponse:
    def __init__(self, model: AsyncModelResource) -> None:
        self._model = model

        self.create = async_to_raw_response_wrapper(
            model.create,
        )
        self.delete = async_to_raw_response_wrapper(
            model.delete,
        )

    @cached_property
    def info(self) -> AsyncInfoResourceWithRawResponse:
        return AsyncInfoResourceWithRawResponse(self._model.info)

    @cached_property
    def update(self) -> AsyncUpdateResourceWithRawResponse:
        return AsyncUpdateResourceWithRawResponse(self._model.update)


class ModelResourceWithStreamingResponse:
    def __init__(self, model: ModelResource) -> None:
        self._model = model

        self.create = to_streamed_response_wrapper(
            model.create,
        )
        self.delete = to_streamed_response_wrapper(
            model.delete,
        )

    @cached_property
    def info(self) -> InfoResourceWithStreamingResponse:
        return InfoResourceWithStreamingResponse(self._model.info)

    @cached_property
    def update(self) -> UpdateResourceWithStreamingResponse:
        return UpdateResourceWithStreamingResponse(self._model.update)


class AsyncModelResourceWithStreamingResponse:
    def __init__(self, model: AsyncModelResource) -> None:
        self._model = model

        self.create = async_to_streamed_response_wrapper(
            model.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            model.delete,
        )

    @cached_property
    def info(self) -> AsyncInfoResourceWithStreamingResponse:
        return AsyncInfoResourceWithStreamingResponse(self._model.info)

    @cached_property
    def update(self) -> AsyncUpdateResourceWithStreamingResponse:
        return AsyncUpdateResourceWithStreamingResponse(self._model.update)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hanzoai import Hanzo, AsyncHanzo
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestContent:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: Hanzo) -> None:
        content = client.files.content.retrieve(
            file_id="file_id",
            provider="provider",
        )
        assert_matches_type(object, content, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: Hanzo) -> None:
        response = client.files.content.with_raw_response.retrieve(
            file_id="file_id",
            provider="provider",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        content = response.parse()
        assert_matches_type(object, content, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: Hanzo) -> None:
        with client.files.content.with_streaming_response.retrieve(
            file_id="file_id",
            provider="provider",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            content = response.parse()
            assert_matches_type(object, content, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: Hanzo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `provider` but received ''"):
            client.files.content.with_raw_response.retrieve(
                file_id="file_id",
                provider="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `file_id` but received ''"):
            client.files.content.with_raw_response.retrieve(
                file_id="",
                provider="provider",
            )


class TestAsyncContent:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncHanzo) -> None:
        content = await async_client.files.content.retrieve(
            file_id="file_id",
            provider="provider",
        )
        assert_matches_type(object, content, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncHanzo) -> None:
        response = await async_client.files.content.with_raw_response.retrieve(
            file_id="file_id",
            provider="provider",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        content = await response.parse()
        assert_matches_type(object, content, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncHanzo) -> None:
        async with async_client.files.content.with_streaming_response.retrieve(
            file_id="file_id",
            provider="provider",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            content = await response.parse()
            assert_matches_type(object, content, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncHanzo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `provider` but received ''"):
            await async_client.files.content.with_raw_response.retrieve(
                file_id="file_id",
                provider="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `file_id` but received ''"):
            await async_client.files.content.with_raw_response.retrieve(
                file_id="",
                provider="provider",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["LiteLlmModelTable"]


class LiteLlmModelTable(BaseModel):
    created_by: str

    updated_by: str

    api_model_aliases: Union[str, object, None] = FieldInfo(alias="model_aliases", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .team import (
    TeamResource,
    AsyncTeamResource,
    TeamResourceWithRawResponse,
    AsyncTeamResourceWithRawResponse,
    TeamResourceWithStreamingResponse,
    AsyncTeamResourceWithStreamingResponse,
)
from .model import (
    ModelResource,
    AsyncModelResource,
    ModelResourceWithRawResponse,
    AsyncModelResourceWithRawResponse,
    ModelResourceWithStreamingResponse,
    AsyncModelResourceWithStreamingResponse,
)
from .callback import (
    CallbackResource,
    AsyncCallbackResource,
    CallbackResourceWithRawResponse,
    AsyncCallbackResourceWithRawResponse,
    CallbackResourceWithStreamingResponse,
    AsyncCallbackResourceWithStreamingResponse,
)

__all__ = [
    "ModelResource",
    "AsyncModelResource",
    "ModelResourceWithRawResponse",
    "AsyncModelResourceWithRawResponse",
    "ModelResourceWithStreamingResponse",
    "AsyncModelResourceWithStreamingResponse",
    "CallbackResource",
    "AsyncCallbackResource",
    "CallbackResourceWithRawResponse",
    "AsyncCallbackResourceWithRawResponse",
    "CallbackResourceWithStreamingResponse",
    "AsyncCallbackResourceWithStreamingResponse",
    "TeamResource",
    "AsyncTeamResource",
    "TeamResourceWithRawResponse",
    "AsyncTeamResourceWithRawResponse",
    "TeamResourceWithStreamingResponse",
    "AsyncTeamResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["TeamListParams"]


class TeamListParams(TypedDict, total=False):
    organization_id: Optional[str]

    user_id: Optional[str]
    """Only return teams which this 'user_id' belongs to"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .cancel import (
    CancelResource,
    AsyncCancelResource,
    CancelResourceWithRawResponse,
    AsyncCancelResourceWithRawResponse,
    CancelResourceWithStreamingResponse,
    AsyncCancelResourceWithStreamingResponse,
)
from .batches import (
    BatchesResource,
    AsyncBatchesResource,
    BatchesResourceWithRawResponse,
    AsyncBatchesResourceWithRawResponse,
    BatchesResourceWithStreamingResponse,
    AsyncBatchesResourceWithStreamingResponse,
)

__all__ = [
    "CancelResource",
    "AsyncCancelResource",
    "CancelResourceWithRawResponse",
    "AsyncCancelResourceWithRawResponse",
    "CancelResourceWithStreamingResponse",
    "AsyncCancelResourceWithStreamingResponse",
    "BatchesResource",
    "AsyncBatchesResource",
    "BatchesResourceWithRawResponse",
    "AsyncBatchesResourceWithRawResponse",
    "BatchesResourceWithStreamingResponse",
    "AsyncBatchesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["UtilTransformRequestResponse"]


class UtilTransformRequestResponse(BaseModel):
    error: Optional[str] = None

    raw_request_api_base: Optional[str] = None

    raw_request_body: Optional[object] = None

    raw_request_headers: Optional[object] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hanzoai import Hanzo, AsyncHanzo
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCallback:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: Hanzo) -> None:
        callback = client.team.callback.retrieve(
            "team_id",
        )
        assert_matches_type(object, callback, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: Hanzo) -> None:
        response = client.team.callback.with_raw_response.retrieve(
            "team_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        callback = response.parse()
        assert_matches_type(object, callback, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: Hanzo) -> None:
        with client.team.callback.with_streaming_response.retrieve(
            "team_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            callback = response.parse()
            assert_matches_type(object, callback, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: Hanzo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `team_id` but received ''"):
            client.team.callback.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_add(self, client: Hanzo) -> None:
        callback = client.team.callback.add(
            team_id="team_id",
            callback_name="callback_name",
            callback_vars={"foo": "string"},
        )
        assert_matches_type(object, callback, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_add_with_all_params(self, client: Hanzo) -> None:
        callback = client.team.callback.add(
            team_id="team_id",
            callback_name="callback_name",
            callback_vars={"foo": "string"},
            callback_type="success",
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(object, callback, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_add(self, client: Hanzo) -> None:
        response = client.team.callback.with_raw_response.add(
            team_id="team_id",
            callback_name="callback_name",
            callback_vars={"foo": "string"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        callback = response.parse()
        assert_matches_type(object, callback, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_add(self, client: Hanzo) -> None:
        with client.team.callback.with_streaming_response.add(
            team_id="team_id",
            callback_name="callback_name",
            callback_vars={"foo": "string"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            callback = response.parse()
            assert_matches_type(object, callback, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_add(self, client: Hanzo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `team_id` but received ''"):
            client.team.callback.with_raw_response.add(
                team_id="",
                callback_name="callback_name",
                callback_vars={"foo": "string"},
            )


class TestAsyncCallback:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncHanzo) -> None:
        callback = await async_client.team.callback.retrieve(
            "team_id",
        )
        assert_matches_type(object, callback, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncHanzo) -> None:
        response = await async_client.team.callback.with_raw_response.retrieve(
            "team_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        callback = await response.parse()
        assert_matches_type(object, callback, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncHanzo) -> None:
        async with async_client.team.callback.with_streaming_response.retrieve(
            "team_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            callback = await response.parse()
            assert_matches_type(object, callback, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncHanzo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `team_id` but received ''"):
            await async_client.team.callback.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_add(self, async_client: AsyncHanzo) -> None:
        callback = await async_client.team.callback.add(
            team_id="team_id",
            callback_name="callback_name",
            callback_vars={"foo": "string"},
        )
        assert_matches_type(object, callback, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_add_with_all_params(self, async_client: AsyncHanzo) -> None:
        callback = await async_client.team.callback.add(
            team_id="team_id",
            callback_name="callback_name",
            callback_vars={"foo": "string"},
            callback_type="success",
            litellm_changed_by="litellm-changed-by",
        )
        assert_matches_type(object, callback, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_add(self, async_client: AsyncHanzo) -> None:
        response = await async_client.team.callback.with_raw_response.add(
            team_id="team_id",
            callback_name="callback_name",
            callback_vars={"foo": "string"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        callback = await response.parse()
        assert_matches_type(object, callback, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_add(self, async_client: AsyncHanzo) -> None:
        async with async_client.team.callback.with_streaming_response.add(
            team_id="team_id",
            callback_name="callback_name",
            callback_vars={"foo": "string"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            callback = await response.parse()
            assert_matches_type(object, callback, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_add(self, async_client: AsyncHanzo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `team_id` but received ''"):
            await async_client.team.callback.with_raw_response.add(
                team_id="",
                callback_name="callback_name",
                callback_vars={"foo": "string"},
            )
